/* copyright (c)2012 Oskar Schirmer <oskar@scara.com> */
#include "chanend.h"
#include "print.h"
#include "reception.h"
#include "resource.h"
#include "uart.h"

static void dumpreg_procstat(resource log, unsigned reg)
{
  unsigned j = ps_get(reg);
  printstr(log, "procstat ");
  printhex(log, reg, 2);
  printstr(log, ": ");
  printhex(log, j, 8);
  println(log);
}

static void dumpreg_dest(resource log, char *name, resource dst,
        unsigned reg, unsigned num)
{
  resource c;
  unsigned i, j;
  printstr(log, name);
  printchar(log, ' ');
  printhex(log, reg, 2);
  printchar(log, ':');
  c = chanend_book();
  for (i = 0; i < num; i++) {
    chanend_destination(c, dst);
    __asm__ __volatile__ (
      "ldc r11, 0xc1; outct res[%0], r11" : : "r" (c) : "r11");
    chanend_write_byte(c, c >> 24);
    chanend_write_byte(c, c >> 16);
    chanend_write_byte(c, c >> 8);
    chanend_write_byte(c, (reg + i) >> 8);
    chanend_write_byte(c, reg + i);
    chanend_all_sent(c);
    __asm__ __volatile__ (
      "chkct res[%0], 3" : : "r" (c));
    j = chanend_read_word(c);
    chanend_all_received(c);
    printchar(log, ' ');
    printhex(log, j, 8);
  }
  chanend_free(c);
  println(log);
}

static void dumpreg_ps(resource log, unsigned core, unsigned reg, unsigned num)
{
  dumpreg_dest(log, "PS", 0x0000c20c + (core << 16), reg, num);
}

static void dumpreg_ic(resource log, unsigned reg, unsigned num)
{
  dumpreg_dest(log, "IC", 0x0000c30c, reg, num);
}

void dumpreg(void)
{
  resource log;
  log = chanend_book();
  chanend_destination(log, chanend_cr_request(log, cr_uart_tx));
  dumpreg_procstat(log, 0);
  dumpreg_procstat(log, 1);
  dumpreg_procstat(log, 3);
  dumpreg_ps(log, log >> 16, 0x00, 3);
  dumpreg_ps(log, log >> 16, 0x04, 4);
  dumpreg_ps(log, log >> 16, 0x10, 4);
  dumpreg_ps(log, log >> 16, 0x20, 8);
  dumpreg_ps(log, log >> 16, 0x40, 8);
  dumpreg_ps(log, log >> 16, 0x60, 8);
  dumpreg_ps(log, log >> 16, 0x80, 8);
  dumpreg_ps(log, log >> 16, 0x88, 8);
  dumpreg_ps(log, log >> 16, 0x90, 8);
  dumpreg_ps(log, log >> 16, 0x98, 8);
  dumpreg_ic(log, 0x00, 2);
  dumpreg_ic(log, 0x04, 5);
  dumpreg_ic(log, 0x20, 8);
  dumpreg_ic(log, 0x28, 8);
  dumpreg_ic(log, 0x40, 8);
  dumpreg_ic(log, 0x48, 8);
  dumpreg_ic(log, 0x80, 8);
  dumpreg_ic(log, 0x88, 8);
  chanend_free(log);
}
